| Title: | Making "Deduplicated" Functions |
|---|---|
| Description: | Contains one main function deduped() which speeds up slow, vectorized functions by only performing computations on the unique values of the input and expanding the results at the end. |
| Authors: | Or Gadish [aut, cre, cph] |
| Maintainer: | Or Gadish <[email protected]> |
| License: | MIT + file LICENSE |
| Version: | 0.5.0 |
| Built: | 2026-07-05 09:10:22 UTC |
| Source: | https://github.com/orgadish/deduped |
Converts a vectorized function into one that only performs the computations on unique values in the first argument. The result is then expanded so that it is the same as if the computation was performed on all elements.
Note: This only works with functions that preserve length and order.
deduped(f, verbose = NULL)deduped(f, verbose = NULL)
f |
A length-preserving, order-preserving function that accepts a vector or list as its first input. |
verbose |
If |
We make a best effort to preserve the two main cases for named inputs:
If f() drops names, names are dropped in the output.
Otherwise, we preserve the names from the input x.
We cannot reliably re-expand the names from the deduped output, since
duplicate values would always map back to their first occurrence's name.
Deduplicated version of f.
x <- sample(LETTERS, 10) x large_x <- sample(rep(x, 10)) length(large_x) slow_func <- function(x) { for (i in x) { Sys.sleep(0.001) } tolower(x) } system.time({ y1 <- slow_func(large_x) }) system.time({ y2 <- deduped(slow_func)(large_x) }) all(y1 == y2)x <- sample(LETTERS, 10) x large_x <- sample(rep(x, 10)) length(large_x) slow_func <- function(x) { for (i in x) { Sys.sleep(0.001) } tolower(x) } system.time({ y1 <- slow_func(large_x) }) system.time({ y2 <- deduped(slow_func)(large_x) }) all(y1 == y2)
This is a convenience wrapper for deduped() to allow it to be piped into
an expression. It will recursively parse the first arguments of the
expression call tree to find the bottom – when the first argument is not
itself a function call.
Without nesting: f(x, ...) |> with_deduped() is equivalent to
deduped(\(.z) f(.z, ...))(x).
With nesting: f(g(x, g2), f2) |> with_deduped() is equivalent to
deduped(\(.z) f(g(.z, g2), f2))(x).
with_deduped(expr, env = parent.frame(), verbose = NULL)with_deduped(expr, env = parent.frame(), verbose = NULL)
expr |
The expression to evaluate. |
env |
The environment within which to evaluate the expression. Can be modified when calling inside other functions. |
verbose |
If |
with_deduped() reconstructs the wrapper function on every call, so it is
best suited for one-off or interactive use. For repeated calls such as inside
a loop, build the wrapper once with deduped() instead:
# Preferred for loops: deduped_f <- deduped(slow_func) for (...) deduped_f(x) # Rather than: for (...) slow_func(x) |> with_deduped()
The result of evaluating the expression.
x <- sample(LETTERS, 10) x large_x <- sample(rep(x, 10)) length(large_x) slow_func <- function(x) { for (i in x) { Sys.sleep(0.001) } tolower(x) } system.time({ y1 <- slow_func(large_x) }) system.time({ y2 <- with_deduped(slow_func(large_x)) # Can also use the R pipe (R >= 4.1.0) or magrittr pipe, for convenience. # slow_func(large_x) |> with_deduped() }) all(y1 == y2)x <- sample(LETTERS, 10) x large_x <- sample(rep(x, 10)) length(large_x) slow_func <- function(x) { for (i in x) { Sys.sleep(0.001) } tolower(x) } system.time({ y1 <- slow_func(large_x) }) system.time({ y2 <- with_deduped(slow_func(large_x)) # Can also use the R pipe (R >= 4.1.0) or magrittr pipe, for convenience. # slow_func(large_x) |> with_deduped() }) all(y1 == y2)